//
//  DetailsViewController.swift
//  AhoyMap-Example
//
//  Created by Syed Ismail Ahamed on 14/05/2022.
//

import UIKit
import AhoyMaps
import CoreLocation

class DetailsViewController: UIViewController {

    @IBOutlet weak var ahoyMapView: AhoyMapView!
    var polyline: Polyline?
    var polygon: Polygon?
    var marker: Marker?

    var viewModel: DetailsViewModel

    init(viewModel: DetailsViewModel) {
        self.viewModel = viewModel
        super.init(nibName: nil, bundle: nil)
    }
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    override func viewDidDisappear(_ animated: Bool) {
        showFeature()
    }
    override func viewDidLoad() {
        super.viewDidLoad()
        let removeButton = UIBarButtonItem(title: "Remove", style: .plain, target: self, action: #selector(removeTapped))
        self.navigationItem.rightBarButtonItem = removeButton

        ahoyMapView.setGestureDelegates(self)

    }

    @objc func removeTapped() {
        removeFeature()
    }
    override func viewDidAppear(_ animated: Bool) {
        self.showFeature()

    }


}



extension DetailsViewController {

    func showFeature() {
        switch viewModel.feature {
            case .polygon:
                addPolygon()
            case .polyline:
                addPolyline()
            case .marker:
                addMarker()
            case .zoom:
                setZoomLevel(level: 20)
            case .traffic:
                showTraffic(isOn: true)
            case .location:
                moveToLocation()
            case .normalDay:
                showScheme(scheme: .normalDay)
            case .normalNight:
                showScheme(scheme: .normalNight)
            case .satellite:
                showScheme(scheme: .satellite)
            case .hybridDay:
                showScheme(scheme: .hybridDay)
            case .hybridNight:
                showScheme(scheme: .hybridNight)
            case .navigate:
                break

        }
    }

  

    func removeFeature() {
        switch viewModel.feature {
            case .polygon:
                if let polygon = polygon {
                    ahoyMapView.removePolygon(polygon: polygon)
                }
            case .polyline:
                if let polyline = polyline {
                    ahoyMapView.removePolyline(model: polyline)
                }
            case .marker:
                if let marker = marker {
                    ahoyMapView.removeMarker(marker: marker)
                }
            default :
                break
        }
    }
}
extension DetailsViewController {

    //MARK:- Add PolyLine
    func addPolyline() {
        do {
            let geoCoordinatesArray = [
                CLLocationCoordinate2D(latitude: 25.06374149085386, longitude: 55.139550763536),
                CLLocationCoordinate2D(latitude: 25.064042044373057,  longitude: 55.13910269599112),
                CLLocationCoordinate2D(latitude: 25.06439140480202,  longitude: 55.1386659719284)
            ]
            polyline = Polyline(coordinates: geoCoordinatesArray, lineWidth: 30, lineColor: .red)
            guard let polyline = polyline else {
                return
            }
            try ahoyMapView.addPolyLine(polyline: polyline)
        } catch let error {
            print("could not add polyline \(error.localizedDescription)")
        }
    }

    //MARK: - Polygon
    func addPolygon() {
        do {


            
            let geoCoordinatesArray = [
                CLLocationCoordinate2D(latitude: 25.06506535822475, longitude: 55.13878471359273),
                CLLocationCoordinate2D(latitude: 25.064909862571714, longitude: 55.13902074796678),
                CLLocationCoordinate2D(latitude: 25.064671759470496, longitude: 55.13876325592236),
                CLLocationCoordinate2D(latitude: 25.064817536934328, longitude: 55.138554043636255),
                CLLocationCoordinate2D(latitude: 25.065045921278926, longitude: 55.13877934917513)

            ]
            polygon = Polygon(coordinates: geoCoordinatesArray, color: .red)
            guard let polygon = polygon else {
                return
            }
            try ahoyMapView.addPolygon(polygon: polygon)
        } catch let error {
            print("could not add polygon \(error.localizedDescription)")
        }
    }

    //MARK: - Marker
    func addMarker() {
        do {
            let markerLocation = CLLocationCoordinate2D(latitude: 25.064540396457215, longitude: 55.13847880446196)
            marker = Marker(coordinates: markerLocation, image: UIImage(named: "location-marker-icon"))
            guard let marker = marker else {
                return
            }
            try ahoyMapView.addMarker(marker: marker)
        } catch let error {
            print("could not add marker \(error.localizedDescription)")
        }
    }

    //MARK: - Marker
    func setZoomLevel(level: Double) {
        ahoyMapView.setZoomLevel(level: level)
    }

    //MARK: - Traffic
    func showTraffic(isOn: Bool) {
        ahoyMapView.showTrafficFlow(isTrafficOn: isOn)
    }

    //MARK: - Move to Location
    func moveToLocation() {
        let cameraLocation = CLLocationCoordinate2D(latitude: 25.308147142589714, longitude: 55.37651267478113)
        ahoyMapView.moveCamera(coordinates: cameraLocation)
    }

    //MARK: - Show Scheme
    func showScheme(scheme:AhoyMapScheme) {
        ahoyMapView.setMapScheme(scheme: scheme)
    }
}

extension DetailsViewController: AhoyMapViewGestureDelegate {
    func didFinishPanGesture(location: CLLocationCoordinate2D) {

    }

    func onMapIdle() {
    
    }

    func didFinishPanGesture(location: CLLocationCoordinate2D, centerCoordinate: CLLocationCoordinate2D) {

    }

    func didFinishLongPressGesture(location: CLLocationCoordinate2D) {

    }

    func didFinishTapGesture(location: CLLocationCoordinate2D) {

    }


}
